-----------------------------------------------------------------------------------------------
-- Client Lua Script for Revenge
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GameLib"
require "FriendshipLib"
 
-----------------------------------------------------------------------------------------------
-- Revenge Module Definition
-----------------------------------------------------------------------------------------------
Revenge = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local kcrSelectedText = ApolloColor.new("UI_BtnTextHoloPressedFlyby")
local kcrNormalText = ApolloColor.new("UI_BtnTextHoloNormal")
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Revenge:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

	o.tItems = {} -- keep track of all the list items
	o.wndSelectedListItem = nil -- keep track of which list item is currently selected
	
	self._arrayRevengeList = {}
	self._tableRevengeList = {}
	self._tableRivalList = {} --used by unitcreated and destroyed
	self._tableControls = {}
	self._tableOptions = {
		showOnDeath = false,
		attachRivalIcons = false,
	}
	self._tableInternal = {
		sod = false,
		ari = false,
		busy = false,
	}
--	self._unitPlayer = nil
	self._intRivalCount = nil
	self._tableRivalNames = {}

    return o
end

function Revenge:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Revenge"
	local tDependencies = {}
	Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Revenge OnLoad
-----------------------------------------------------------------------------------------------
function Revenge:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("Revenge.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function Revenge:OnSave(saveDepth)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return nil end

	local savedVariables = {}
	for key,value in next, self._tableOptions do
		savedVariables[key] = value
	end

	return savedVariables
end

function Revenge:OnRestore(saveDepth, savedVariables)
	if saveDepth ~= GameLib.CodeEnumAddonSaveLevel.Character then return end	
	if savedVariables == {} or savedVariables == nil then return end
	
	for key,value in next, savedVariables do
		self._tableOptions[key] = value 
	end
end

-----------------------------------------------------------------------------------------------
-- Revenge OnDocLoaded
-----------------------------------------------------------------------------------------------
function Revenge:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "RevengeForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.wndItemList = self.wndMain:FindChild("ItemList")
		self.wndMain:Show(false, true)

		--self.xmlDoc = nil
		
		Apollo.RegisterEventHandler("CombatLogAbsorption", 				"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogCCState", 				"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogDamage", 				"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogDispel", 					"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogModifyInterruptArmor", 		"OnCombatAction", self)
--		Apollo.RegisterEventHandler("CombatLogTransference", 			"OnCombatAction", self)
--		Apollo.RegisterEventHandler("CombatLogVitalModifier", 			"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogDeflect", 				"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogImmunity", 				"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogInterrupted", 				"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogKillPVP", 				"OnCombatAction", self)
--		Apollo.RegisterEventHandler("CombatLogLifeSteal", 				"OnCombatAction", self)
		Apollo.RegisterEventHandler("CombatLogDeath",					"OnDeath", self)
		
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", 			"OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("ToggleRevengeWindow", 				"OnRevengeOn", self)
		
		Apollo.RegisterEventHandler("UnitCreated", 						"OnUnitCreated", self)
		Apollo.RegisterEventHandler("UnitDestroyed", 					"OnUnitDestroyed", self)
		
		Apollo.RegisterEventHandler("MatchJoined",						"OnBusy", self)
		Apollo.RegisterEventHandler("MatchLeft",						"OnBusy", self)

		Apollo.RegisterEventHandler("ChangeWorld", 					"OnChangeWorld", self)
		
		self._delayedStart = ApolloTimer.Create(0.5, true, 				"OnDelayedStart", self)
		
		Apollo.RegisterSlashCommand("revenge", 						"OnRevengeOn", self)
	end
end

-----------------------------------------------------------------------------------------------
-- RevengeForm Functions
-----------------------------------------------------------------------------------------------
function Revenge:OnCombatAction(tEventArgs)
	if self._tableInternal.busy then return end
	
	local player = self._unitPlayer
	local caster = tEventArgs.unitCaster
	local target = tEventArgs.unitTarget
	
	if caster == nil or not caster:IsACharacter() or target ~= player or target == caster or caster:IsRival() then return end --not interested if not a player, or already a rival 
	
	local oppressor = caster:GetName()
	if not self._tableRevengeList[oppressor] then
		self._tableRevengeList[oppressor] = true
		table.insert(self._arrayRevengeList, 1, oppressor)
	-- elseif self._tableRevengeList[oppressor] and self._arrayRevengeList[1] ~= oppressor then
		-- for index, iname in next, self._arrayRevengeList do if iname == oppressor then table.remove(self._arrayRevengeList, index) end end
		-- table.insert(self._arrayRevengeList, 1, oppressor)
	end
end

function Revenge:OnDelayedStart() --Y U NO WORK
	self._unitPlayer = GameLib.GetPlayerUnit()
	if not self._unitPlayer then return end
	
	self._delayedStart:Stop() 
end

function Revenge:OnChangeWorld()
	self._unitPlayer = nil
	self._delayedStart:Start()
end

function Revenge:OnDeath(tEventArgs)
	if self._tableInternal.busy then return end
	
	if self._tableOptions.showOnDeath then self:OnRevengeOn() end
end

function Revenge:OnBusy()
	self._tableInternal.busy = not self._tableInternal.busy
end

function Revenge:OnUnitCreated(unit)
	if not unit or not unit:IsRival() then return end
	
	local id = unit:GetId()
	local name = unit:GetName()

	if not self._tableRivalList[id] then
		self._tableRivalList[id] = true
		table.insert(self._tableRivalNames, "-"..name)
	end
	
	self:UpdateInterfaceMenuAlerts(#self._tableRivalNames)
	
	-- if not self._tableOptions.attachRivalIcons then return end
	
	-- if unit:IsACharacter() and unit:IsRival() then 
		-- local revengeIcon = Apollo.LoadForm(self.xmlDoc, "Rival", "InWorldHudStratum", self)
		-- revengeIcon:SetUnit(unit, 0)
		-- revengeIcon:SetText(unit:GetName())
	-- end
end

function Revenge:OnUnitDestroyed(unit)
	if not unit or not unit:IsRival() then return end
	
	local id = unit:GetId()
	local name = unit:GetName()

	if self._tableRivalList[id] then
		self._tableRivalList[id] = nil
		for index, iname in next, self._tableRivalNames do if iname == "-"..name then table.remove(self._tableRivalNames, index) end end
	end
	
	self:UpdateInterfaceMenuAlerts(#self._tableRivalNames)
end

-----------------------------------------------------------------------------------------------
-- RevengeForm Functions
-----------------------------------------------------------------------------------------------
function Revenge:OnRevengeOn()
	self._tableControls.sod_checkbox = self.wndMain:FindChild("AutoShowCheckbox")
	self._tableControls.ari_checkbox = self.wndMain:FindChild("RivalIconsCheckbox")
	
	self._tableControls.sod_checkbox:SetCheck(self._tableOptions.showOnDeath)
	self._tableControls.ari_checkbox:SetCheck(self._tableOptions.attachRivalIcons)
	
	self._tableControls.ari_checkbox:Enable(false)
	
	self._tableInternal.sod = self._tableOptions.showOnDeath
	self._tableInternal.ari = self._tableOptions.attachRivalIcons
	
	self.wndMain:Invoke()
	
	self:PopulateItemList()
end

function Revenge:OnConfigure()
	self:OnRevengeOn()
end

function Revenge:OnOK()
	self._tableOptions.showOnDeath = self._tableControls.sod_checkbox:IsChecked()
	self._tableOptions.attachRivalIcons = self._tableControls.ari_checkbox:IsChecked()
	
	self.wndMain:Close()
	if self._tableInternal.sod ~= self._tableOptions.showOnDeath
	or self._tableInternal.ari ~= self._tableOptions.attachRivalIcons
	then RequestReloadUI() end --request a UI reload in case settings are changed, otherwise do nothing essentially
end

function Revenge:OnCancel()
	self.wndMain:Close()
end

function Revenge:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Revenge", {"ToggleRevengeWindow", "", "IconSprites:Icon_MapNode_Map_Rival"})
end

function Revenge:UpdateInterfaceMenuAlerts(rivalCount)
	rivalCount = rivalCount or 0
	local strRivalCount = (rivalCount == 0) and "No" or rivalCount
	local rivalText = (rivalCount == 1) and " rival nearby\n" or " rivals nearby\n"
	Event_FireGenericEvent("InterfaceMenuList_AlertAddOn", "Revenge", {false, "\n"..strRivalCount..rivalText..table.concat(self._tableRivalNames, "\n"), rivalCount})
end

-----------------------------------------------------------------------------------------------
-- ItemList Functions
-----------------------------------------------------------------------------------------------
function Revenge:PopulateItemList()
	self:DestroyItemList()

	for index, oppressor in next, self._arrayRevengeList do
		self:AddItem(index, oppressor)
	end
	
	self.wndItemList:ArrangeChildrenVert()
end

function Revenge:DestroyItemList()
	for idx,wnd in ipairs(self.tItems) do
		wnd:Destroy()
	end

	self.tItems = {}
	self.wndSelectedListItem = nil
end

function Revenge:AddItem(index, oppressor)
	local wnd = Apollo.LoadForm(self.xmlDoc, "ListItem", self.wndItemList, self)
	
	self.tItems[index] = wnd

	local wndItemText = wnd:FindChild("Text")
	local wndItemAddButton = wnd:FindChild("Add")
	local wndItemIgnoreButton = wnd:FindChild("Ignore")
	if wndItemText then -- make sure the text wnd exist
		wndItemText:SetText(oppressor) -- set the item wnd's text to "item i"
		wndItemAddButton:Show(false)
		wndItemIgnoreButton:Show(false)
		wndItemText:SetTextColor(kcrNormalText)
	end
	wnd:SetData(index)
end

function Revenge:OnListItemSelected(wndHandler, wndControl)
	if wndHandler ~= wndControl then
		return
	end
    
	if self.wndSelectedListItem ~= nil then
		self.wndSelectedListItem:FindChild("Text"):SetTextColor(kcrNormalText)
		self.wndSelectedListItem:FindChild("Add"):FindChild("Add"):Show(false)
		self.wndSelectedListItem:FindChild("Ignore"):FindChild("Ignore"):Show(false)
	end
    
	self.wndSelectedListItem = wndControl
	
	self.wndSelectedListItem:FindChild("Text"):SetTextColor(kcrSelectedText)
	self.wndSelectedListItem:FindChild("Add"):Show(true)
	self.wndSelectedListItem:FindChild("Ignore"):Show(true)
end

---------------------------------------------------------------------------------------------------
-- ListItem Functions
---------------------------------------------------------------------------------------------------

function Revenge:OnAddButtonClicked( wndHandler, wndControl, eMouseButton )
	FriendshipLib.AddByName(FriendshipLib.CharacterFriendshipType_Rival, self.wndSelectedListItem:FindChild("Text"):GetText())
	table.remove(self._arrayRevengeList, self.wndSelectedListItem:GetData())
	self.wndSelectedListItem = nil
	self:PopulateItemList()
end

function Revenge:OnIgnoreButtonClicked( wndHandler, wndControl, eMouseButton )
	table.remove(self._arrayRevengeList, self.wndSelectedListItem:GetData())
	self.wndSelectedListItem = nil
	self:PopulateItemList()
end

-----------------------------------------------------------------------------------------------
-- Revenge Instance
-----------------------------------------------------------------------------------------------
local RevengeInst = Revenge:new()
RevengeInst:Init()
